CREATE OR REPLACE PACKAGE BODY xlib_conf
AS
----------------------------------------------------------------------------
--
----------------------------------------------------------------------------
   FUNCTION get_value (
      p_conf_key   IN   VARCHAR2,
      p_app_ctx    IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain     IN   VARCHAR2 DEFAULT 'GLOBAL'
   )
      RETURN VARCHAR2
   IS
      l_value   xlib_conf_values.conf_value%TYPE;
   BEGIN
      SELECT conf_value
        INTO l_value
        FROM xlib_conf_values
       WHERE conf_key = p_conf_key
         AND conf_app_ctx = p_app_ctx
         AND conf_domain = p_domain;

      RETURN l_value;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN NULL;
   END;

----------------------------------------------------------------------------
--
----------------------------------------------------------------------------
   FUNCTION v (
      p_conf_key   IN   VARCHAR2,
      p_app_ctx    IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain     IN   VARCHAR2 DEFAULT 'GLOBAL'
   )
      RETURN VARCHAR2
   IS
   BEGIN
      RETURN get_value (p_conf_key      => p_conf_key,
                        p_app_ctx       => p_app_ctx,
                        p_domain        => p_domain
                       );
   END;

----------------------------------------------------------------------------
--
----------------------------------------------------------------------------
   FUNCTION nv (
      p_conf_key     IN   VARCHAR2,
      p_num_format   IN   VARCHAR2 DEFAULT NULL,
      p_app_ctx      IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain       IN   VARCHAR2 DEFAULT 'GLOBAL'
   )
      RETURN NUMBER
   IS
   BEGIN
      IF p_num_format IS NULL
      THEN
         RETURN TO_NUMBER (get_value (p_conf_key      => p_conf_key,
                                      p_app_ctx       => p_app_ctx,
                                      p_domain        => p_domain
                                     )
                          );
      ELSE
         RETURN TO_NUMBER (get_value (p_conf_key      => p_conf_key,
                                      p_app_ctx       => p_app_ctx,
                                      p_domain        => p_domain
                                     ),
                           p_num_format
                          );
      END IF;
   END;

----------------------------------------------------------------------------
--
----------------------------------------------------------------------------
   FUNCTION dv (
      p_conf_key      IN   VARCHAR2,
      p_date_format   IN   VARCHAR2 DEFAULT NULL,
      p_app_ctx       IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain        IN   VARCHAR2 DEFAULT 'GLOBAL'
   )
      RETURN DATE
   IS
   BEGIN
      IF p_date_format IS NULL
      THEN
         RETURN TO_DATE (get_value (p_conf_key      => p_conf_key,
                                    p_app_ctx       => p_app_ctx,
                                    p_domain        => p_domain
                                   )
                        );
      ELSE
         RETURN TO_DATE (get_value (p_conf_key      => p_conf_key,
                                    p_app_ctx       => p_app_ctx,
                                    p_domain        => p_domain
                                   ),
                         p_date_format
                        );
      END IF;
   END;

----------------------------------------------------------------------------
--
----------------------------------------------------------------------------
   PROCEDURE set_value (
      p_conf_key     IN   VARCHAR2,
      p_conf_value   IN   VARCHAR2,
      p_app_ctx      IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain       IN   VARCHAR2 DEFAULT 'GLOBAL',
	  p_comment      in   varchar2 default null
   )
   IS
   BEGIN
      -- update key
      UPDATE xlib_conf_values
         SET conf_value = p_conf_value
       WHERE conf_key = p_conf_key
         AND conf_app_ctx = p_app_ctx
         AND conf_domain = p_domain;

      -- key not found, create key
      IF SQL%ROWCOUNT = 0
      THEN
         INSERT INTO xlib_conf_values
                     (conf_key, conf_value, conf_app_ctx, conf_domain, conf_comment
                     )
              VALUES (p_conf_key, p_conf_value, p_app_ctx, p_domain,p_comment
                     );
      END IF;
   END;

----------------------------------------------------------------------------
-- deletes a conf key from the table xlib_conf_values
----------------------------------------------------------------------------
   PROCEDURE remove_conf_key (
      p_conf_key   IN   VARCHAR2,
      p_app_ctx    IN   VARCHAR2 DEFAULT xlib_context.get_application_context,
      p_domain     IN   VARCHAR2 DEFAULT 'GLOBAL'
   )
   IS
   BEGIN
      DELETE FROM xlib_conf_values
            WHERE conf_key = p_conf_key
              AND conf_app_ctx = p_app_ctx
              AND conf_domain = p_domain;

      IF SQL%ROWCOUNT = 0
      THEN
         RAISE NO_DATA_FOUND;
      END IF;
   END;
END;
/